from .base import const
from .font import Font, FontFamily, FontSize
from .view import View

class Button(View):
    OBJC_CLASS = 'NSButton'
    
    LAYOUT_DELTA_X = -6
    LAYOUT_DELTA_Y = -8
    LAYOUT_DELTA_W = 12
    LAYOUT_DELTA_H = 12
    
    DEFAULT_FONT = Font(FontFamily.System, FontSize.RegularControl)
    
    def __init__(self, parent, title, action=None):
        View.__init__(self, parent, 80, 20)
        self.title = title
        self.action = action
        self.font = self.DEFAULT_FONT
    
    def dependencies(self):
        return [self.font]
    
    def generateInit(self):
        tmpl = View.generateInit(self)
        self.properties['title'] = self.title
        self.properties['font'] = self.font
        self.properties['buttonType'] = const.NSMomentaryLightButton
        self.properties['bezelStyle'] = const.NSRoundedBezelStyle
        tmpl.viewsetup = "$linkaction$\n"
        if self.action:
            tmpl.linkaction = self.action.generate(self.varname)
        else:
            tmpl.linkaction = ''
        return tmpl
    



from argparse import ArgumentParser

from .gen import generate

def main():
    parser = ArgumentParser()
    parser.add_argument('source',
        help="Path of the UI script to convert")
    parser.add_argument('dest',
        help="Destination path for the resulting Objective-C file")
    args = parser.parse_args()
    generate(args.source, args.dest)



from .view import View

class Window(View):
    OBJC_CLASS = 'NSWindow'
    
    def __init__(self, x, y, width, height, title):
        View.__init__(self, None, width, height)
        self.x = x
        self.y = y
        self.title = title
    
    def generateInit(self):
        tmpl = View.generateInit(self)
        tmpl.allocinit = """
            NSWindow *$varname$ = [[NSWindow alloc] initWithContentRect:$rect$ styleMask:$style$
                backing:NSBackingStoreBuffered defer:NO];
        """
        tmpl.style = "NSTitledWindowMask | NSClosableWindowMask | NSMiniaturizableWindowMask | NSResizableWindowMask"
        self.properties['title'] = self.title
        # Windows don't have autoresizingMask and because it's set in View, we have to remove it.
        del self.properties['autoresizingMask']
        return tmpl
    



from __future__ import division

from collections import namedtuple, defaultdict

from .base import GeneratedItem, Literal

class Pack(object):
    # Corners
    UpperLeft = 1
    UpperRight = 2
    LowerLeft = 3
    LowerRight = 4

    # Sides / Align
    Left = 5
    Right = 6
    Above = 7
    Below = 8
    Middle = 9
    
    @staticmethod
    def oppositeSide(side):
        if side == Pack.Left:
            return Pack.Right
        elif side == Pack.Right:
            return Pack.Left
        elif side == Pack.Above:
            return Pack.Below
        elif side == Pack.Below:
            return Pack.Above

Anchor = namedtuple('Anchor', 'corner growX growY')

class View(GeneratedItem):
    OBJC_CLASS = 'NSView'
    
    BORDER_MARGIN = 20
    INTER_VIEW_MARGIN = 8
    # About coordinates: The coordinates below are "Layout coordinates". They will be slightly
    # adjusted at generation time.
    # According to http://www.cocoabuilder.com/archive/cocoa/192607-interface-builder-layout-versus-frame.html
    # the difference between "Frame Rectangle" and "Layout Rectangle" are hardcoded in IB, so we
    # need to maintain our own hardcoded constants for each supported widget.
    LAYOUT_DELTA_X = 0
    LAYOUT_DELTA_Y = 0
    LAYOUT_DELTA_W = 0
    LAYOUT_DELTA_H = 0
    
    def __init__(self, parent, width, height):
        GeneratedItem.__init__(self)
        self.parent = parent
        self.width = width
        self.height = height
        self.x = 0
        self.y = 0
        self.anchor = Anchor(Pack.UpperLeft, False, False)
        # a mapping PackingSide: {views} which is used in fill() to know how much we can fill
        self.neighbors = defaultdict(set)
    
    #--- Pack
    def packToCorner(self, corner):
        assert self.parent is not None
        px, py, pw, ph = self.parent.rect
        x, y, w, h = self.rect
        margin = self.BORDER_MARGIN
        if corner in (Pack.LowerLeft, Pack.UpperLeft):
            x = margin
        else:            
            x = pw - margin - w
        if corner in (Pack.LowerLeft, Pack.LowerRight):
            y = margin
        else:            
            y = ph - margin - h
        self.x, self.y = x, y
    
    def packRelativeTo(self, other, side, align):
        assert other.parent is self.parent
        ox, oy, ow, oh = other.rect
        x, y, w, h = self.rect
        margin = self.INTER_VIEW_MARGIN
        
        if side in (Pack.Above, Pack.Below):
            if align == Pack.Left:
                x = ox
            elif align == Pack.Right:
                x = ox + ow - w
            else:
                x = ox + ((ow - w) / 2)
        elif side == Pack.Left:
            x = ox - margin - w
        else:
            x = ox + ow + margin
        if side in (Pack.Left, Pack.Right):
            if align == Pack.Below:
                y = oy
            elif align == Pack.Above:
                y = oy + oh - h
            else:
                y = oy + ((oh - h) / 2)
        elif side == Pack.Above:
            y = oy + oh + margin
        else:
            y = oy - margin - h
        self.x, self.y = x, y
        self.neighbors[Pack.oppositeSide(side)].add(other)
        other.neighbors[side].add(self)
    
    def setAnchor(self, corner, growX=False, growY=False):
        self.anchor = Anchor(corner, growX, growY)
    
    def fill(self, side):
        assert self.parent is not None
        px, py, pw, ph = self.parent.rect
        x, y, w, h = self.rect
        neighbors = self.neighbors[side]
        margin = self.BORDER_MARGIN
        if side == Pack.Right:
            nx = max([(n.x + n.width) for n in neighbors] + [x+w])
            goal = pw - margin
            growby = goal - nx
            w += growby
            for n in neighbors:
                n.x += growby
        elif side == Pack.Left:
            nx = min([n.x for n in neighbors] + [x])
            goal = margin
            growby = nx - goal
            w += growby
            x -= growby
            for n in neighbors:
                n.x -= growby
        else:
            raise Exception("Vertical fill not supported yet")
        self.x, self.y, self.width, self.height = x, y, w, h
    
    #--- Generate
    def generateInit(self):
        tmpl = GeneratedItem.generateInit(self)
        tmpl.setup = "$viewsetup$\n$addtoparent$\n"
        tmpl.allocinit = "$classname$ *$varname$ = [[$classname$ alloc] initWithFrame:$rect$];"
        x, y, w, h = self.x, self.y, self.width, self.height
        x += self.LAYOUT_DELTA_X
        y += self.LAYOUT_DELTA_Y
        w += self.LAYOUT_DELTA_W
        h += self.LAYOUT_DELTA_H
        tmpl.rect = "NSMakeRect(%d, %d, %d, %d)" % (x, y, w, h)
        if self.anchor.growX and self.anchor.growY:
            resizeMask = 'NSViewWidthSizable|NSViewHeightSizable'
        elif self.anchor.growX:
            if self.anchor.corner in (Pack.LowerLeft, Pack.LowerRight):
                resizeMask = 'NSViewWidthSizable|NSViewMaxYMargin'
            else:
                resizeMask = 'NSViewWidthSizable|NSViewMinYMargin'
        elif self.anchor.growY:
            if self.anchor.corner in (Pack.UpperLeft, Pack.LowerLeft):
                resizeMask = 'NSViewHeightSizable|NSViewMaxXMargin'
            else:
                resizeMask = 'NSViewHeightSizable|NSViewMinXMargin'
        else:
            if self.anchor.corner == Pack.LowerLeft:
                resizeMask = 'NSViewMaxXMargin|NSViewMaxYMargin'
            elif self.anchor.corner == Pack.UpperRight:
                resizeMask = 'NSViewMinXMargin|NSViewMinYMargin'
            elif self.anchor.corner == Pack.LowerRight:
                resizeMask = 'NSViewMinXMargin|NSViewMaxYMargin'
            else:
                resizeMask = 'NSViewMaxXMargin|NSViewMinYMargin'
        self.properties['autoresizingMask'] = Literal(resizeMask)
        if self.parent is not None:
            tmpl.addtoparent = self.parent.generateAddSubview(self)
        return tmpl
    
    def generateAddSubview(self, subview):
        return "[[%s contentView] addSubview:%s];\n" % (self.varname, subview.varname)
    
    @property
    def rect(self):
        return self.x, self.y, self.width, self.height



import re
from collections import defaultdict

try:
    basestring
except NameError: # python 3
    basestring = str

def upFirstLetter(s):
    return s[0].upper() + s[1:]

class CodeTemplate(object):
    def __init__(self, template):
        self._template = template
        self._replacements = {}
    
    def __setattr__(self, key, value):
        if key in ['_template', '_replacements']:
            return object.__setattr__(self, key, value)
        self._replacements[key] = value
    
    def render(self):
        # Because we generate code and that code is likely to contain "{}" braces, it's better if we
        # use more explicit placeholders than the typecal format() method. These placeholders are
        # $name$.
        result = self._template
        replacements = self._replacements
        placeholders = re.findall(r"\$\w+?\$", result)
        while placeholders:
            # We run replacements multiple times because it's possible that one of our replacement
            # strings contain replacement placeholders. We want to perform replacements on those
            # strings too.
            for placeholder in placeholders:
                replacement = replacements.get(placeholder[1:-1], '')
                result = result.replace(placeholder, replacement)
            placeholders = re.findall(r"\$\w+?\$", result)
        return result

class KeyValueId(object):
    # When we set an KeyValueId attribute in our source file, there no convenient way of saying,
    # at the codegen phase "this is exactly when this value was set, so I'll insert code to assign
    # this value here." What we can do, however, is having a dictionary of all keys a certain value
    # was assigned to and when we create the code for that value, we insert assignments right after.
    VALUE2KEYS = defaultdict(set)
    def __init__(self, parent, name, fakeParent=False):
        self._parent = parent
        self._name = name
        # set fakeParent to True when you want to ignore this KeyValueId in accessors. You can use
        # this for stuff like "const.NSOnState" where we want the accessor to be "NSOnState", not
        # [const NSOnState];
        self._fakeParent = fakeParent
        self._children = {}
    
    def __getattr__(self, name):
        if name.startswith('_'):
            return object.__getattribute__(self, name)
        if name in self._children:
            result = self._children[name]
        else:
            result = KeyValueId(self, name)
            self._children[name] = result
        return result
    
    def __setattr__(self, name, value):
        if name.startswith('_'):
            object.__setattr__(self, name, value)
            return
        key = getattr(self, name)
        KeyValueId.VALUE2KEYS[value].add(key)
    
    # the methods below aren't actually private, it's just that we prepend them with underscores to
    # avoid name clashes.
    def _dottedAccessor(self):
        if self._parent and not self._parent._fakeParent:
            return '%s.%s' % (self._parent._dottedAccessor(), self._name)
        else:
            return self._name
    
    def _objcAccessor(self):
        if self._parent and not self._parent._fakeParent:
            return '[%s %s]' % (self._parent._objcAccessor(), self._name)
        else:
            return self._name

owner = KeyValueId(None, 'owner')
NSApp = KeyValueId(None, 'NSApp')
const = KeyValueId(None, 'const', fakeParent=True)

class Action(object):
    def __init__(self, target, selector):
        self.target = target
        self.selector = selector
    
    def generate(self, sender):
        tmpl = CodeTemplate("""[$sender$ setTarget:$target$];
        [$sender$ setAction:$selector$];
        """)
        tmpl.sender = sender
        tmpl.selector = "@selector(%s)" % self.selector
        if self.target:
            tmpl.target = self.target._objcAccessor()
        else:
            tmpl.target = 'nil'
        return tmpl.render()
    

class KeyShortcut(object):
    def __init__(self, shortcutStr):
        self.shortcutStr = shortcutStr
        elements = set(shortcutStr.lower().split('+'))
        flags = []
        availableFlags = [
            ('cmd', 'NSCommandKeyMask'),
            ('ctrl', 'NSControlKeyMask'),
            ('alt', 'NSAlternateKeyMask'),
            ('shift', 'NSShiftKeyMask'),
        ]
        for ident, flag in availableFlags:
            if ident in elements:
                elements.remove(ident)
                flags.append(flag)
        self.flags = '|'.join(flags)
        assert len(elements) == 1
        self.key = list(elements)[0]
        

# Use this in properties when you need it to be generated as-is, and not wrapped as a normal string
class Literal(object):
    def __init__(self, value):
        self.value = value
    

class GeneratedItem(object):
    CREATION_ORDER_COUNTER = 0
    OBJC_CLASS = 'NSObject'
    
    def __init__(self):
        self.creationOrder = GeneratedItem.CREATION_ORDER_COUNTER
        GeneratedItem.CREATION_ORDER_COUNTER += 1
        self.generated = False
        # In case we are never assigned to a top level variable and thus never given a varname
        self.varname = "_tmp%d" % self.creationOrder
        # properties to be set at generation time. For example, if "editable" is set to False,
        # a "[$varname$ setEditable:NO];" statement will be generated.
        self.properties = {}
    
    #--- Virtual
    def generateInit(self):
        tmpl = CodeTemplate("$allocinit$\n$setup$\n$setprop$\n")
        tmpl.varname = self.varname
        tmpl.classname = self.OBJC_CLASS
        tmpl.allocinit = "$classname$ *$varname$ = [[$classname$ alloc] init];"
        tmpl.setup = ''
        return tmpl
    
    def dependencies(self):
        # Return a list of items on which self depends. We'll make sure that they're generated first.
        return []
    
    #--- Public
    def generateAssignments(self):
        if self not in KeyValueId.VALUE2KEYS:
            return ""
        assignments = []
        for key in KeyValueId.VALUE2KEYS[self]:
            parentAccessor = key._parent._objcAccessor()
            setmethod = 'set' + key._name[0].upper() + key._name[1:]
            assignment = "[%s %s: %s];" % (parentAccessor, setmethod, self.varname)
            assignments.append(assignment)
        return '\n'.join(assignments)
    
    def generate(self, *args, **kwargs):
        result = ''
        for dependency in self.dependencies():
            if isinstance(dependency, GeneratedItem) and not dependency.generated:
                result += dependency.generate()
        inittmpl = self.generateInit(*args, **kwargs)
        setprop = ''
        for key, value in self.properties.items():
            if value is None:
                continue
            if isinstance(value, GeneratedItem):
                value = value.varname
            elif isinstance(value, KeyValueId):
                value = value._objcAccessor()
            elif isinstance(value, Literal):
                value = value.value
            elif isinstance(value, basestring):
                value = '@"%s"' % value
            elif isinstance(value, bool):
                value = 'YES' if value else 'NO'
            elif isinstance(value, (int, float)):
                value = str(value)
            else:
                raise TypeError("Can't figure out the property's type")
            setprop += '[$varname$ set%s:%s];\n' % (upFirstLetter(key), value)
        inittmpl.setprop = setprop
        result += inittmpl.render()
        result += self.generateAssignments()
        self.generated = True
        return result
    



from .view import View
from .font import Font, FontFamily, FontSize

class Label(View):
    OBJC_CLASS = 'NSTextField'
    
    LAYOUT_DELTA_X = -3
    LAYOUT_DELTA_W = 6
    
    DEFAULT_FONT = Font(FontFamily.Label, FontSize.RegularControl)
    
    def __init__(self, parent, text):
        View.__init__(self, parent, 100, 17)
        self.text = text
        self.font = self.DEFAULT_FONT
    
    def dependencies(self):
        return [self.font]
    
    def generateInit(self):
        tmpl = View.generateInit(self)
        self.properties['stringValue'] = self.text
        self.properties['font'] = self.font
        self.properties['editable'] = False
        self.properties['selectable'] = False
        self.properties['drawsBackground'] = False
        self.properties['bordered'] = False
        return tmpl
    



from .base import GeneratedItem, KeyShortcut, Literal, Action, NSApp, const

class MenuItem(GeneratedItem):
    OBJC_CLASS = 'NSMenuItem'
    
    def __init__(self, name, action=None, shortcut=None, tag=None):
        GeneratedItem.__init__(self)
        self.name = name
        self.action = action
        if shortcut and not isinstance(shortcut, KeyShortcut):
            shortcut = KeyShortcut(shortcut)
        self.shortcut = shortcut
        self.tag = tag
    
    def generateInit(self, menuname):
        tmpl = GeneratedItem.generateInit(self)
        if self.name == "-":
            tmpl.allocinit = "[$menuname$ addItem:[NSMenuItem separatorItem]];\n"
            tmpl.setup = ""
        else:
            tmpl.allocinit = "NSMenuItem *$varname$ = [$menuname$ addItemWithTitle:@\"$name$\" action:nil keyEquivalent:@\"$key$\"];"
            tmpl.setup = "$linkaction$"
        tmpl.name = self.name
        tmpl.menuname = menuname
        if self.action:
            tmpl.linkaction = self.action.generate(self.varname)
        else:
            tmpl.linkaction = ''
        if self.shortcut:
            tmpl.key = self.shortcut.key
            if self.shortcut.flags:
                self.properties['keyEquivalentModifierMask'] = Literal(self.shortcut.flags)
        else:
            tmpl.key = "nil"
        self.properties['tag'] = self.tag
        return tmpl
    

class Menu(GeneratedItem):
    OBJC_CLASS = 'NSMenu'
    
    def __init__(self, name):
        GeneratedItem.__init__(self)
        self.name = name
        self.items = []
    
    def add(self, menu_or_item, index=None):
        if index is None:
            index = len(self.items)
        self.items.insert(index, menu_or_item)
    
    def addItem(self, name, action=None, shortcut=None, tag=None, index=None):
        item = MenuItem(name, action, shortcut, tag)
        self.add(item, index)
        return item
    
    def addSeparator(self, index=None):
        return self.addItem("-", index=index)
    
    def addMenu(self, name, index=None):
        menu = Menu(name)
        self.add(menu, index)
        return menu
    
    def removeItem(self, index):
        del self.items[index]
    
    def generateInit(self, menuname=None):
        tmpl = GeneratedItem.generateInit(self)
        if menuname:
            tmpl.allocinit = """
                NSMenuItem *_tmpitem = [$menuname$ addItemWithTitle:@"$name$" action:nil keyEquivalent:@""];
                NSMenu *$varname$ = [[[NSMenu alloc] initWithTitle:@"$name$"] autorelease];
                [$menuname$ setSubmenu:$varname$ forItem:_tmpitem];
            """
        else:
            tmpl.allocinit = """
                NSMenu *$varname$ = [[[NSMenu alloc] initWithTitle:@"$name$"] autorelease];
            """
        tmpl.name = self.name
        tmpl.menuname = menuname
        subitemscode = []
        for item in self.items:
            assert isinstance(item, (Menu, MenuItem))
            item.varname = self.varname + '_sub'
            code = item.generate(self.varname)
            # We wrap it in a block to avoid naming clashes.
            subitemscode.append('{' + code + '}')
        tmpl.setup = '\n'.join(subitemscode)
        return tmpl
    

class MainMenu(Menu):
    def __init__(self, appname):
        Menu.__init__(self, "Apple")
        fileMenu = self.addMenu("File")
        editMenu = self.addMenu("Edit")
        windowMenu = self.addMenu("Window")
        helpMenu = self.addMenu("Help")

        fileMenu.addItem("About %s" % appname)
        fileMenu.addSeparator()
        NSApp.servicesMenu = fileMenu.addMenu("Services")
        fileMenu.addSeparator()
        fileMenu.addItem("Hide %s" % appname, Action(NSApp, 'hide:'), 'cmd+h')
        fileMenu.addItem("Hide Others", Action(NSApp, 'hideOtherApplications:'), 'cmd+alt+h')
        fileMenu.addItem("Hide Others", Action(NSApp, 'unhideAllApplications:'))
        fileMenu.addSeparator()
        fileMenu.addItem("Quit %s" % appname, Action(NSApp, 'terminate:'), 'cmd+q')

        editMenu.addItem("Undo", Action(None, 'undo:'), 'cmd+z')
        editMenu.addItem("Redo", Action(None, 'redo:'), 'cmd+shift+z')
        editMenu.addSeparator()
        editMenu.addItem("Cut", Action(None, 'cut:'), 'cmd+x')
        editMenu.addItem("Copy", Action(None, 'copy:'), 'cmd+c')
        editMenu.addItem("Paste", Action(None, 'paste:'), 'cmd+v')
        editMenu.addItem("Paste And Match Style", Action(None, 'pasteAsPlainText:'), 'cmd+alt+shift+v')
        editMenu.addItem("Delete", Action(None, 'delete:'))
        editMenu.addItem("Select All", Action(None, 'selectAll:'), 'cmd+a')
        editMenu.addSeparator()
        findMenu = editMenu.addMenu("Find")
        findMenu.addItem("Find...", Action(None, 'performFindPanelAction:'), 'cmd+f', tag=const.NSFindPanelActionShowFindPanel)
        findMenu.addItem("Find Next", Action(None, 'performFindPanelAction:'), 'cmd+g', tag=const.NSFindPanelActionNext)
        findMenu.addItem("Find Previous", Action(None, 'performFindPanelAction:'), 'cmd+shift+g', tag=const.NSFindPanelActionPrevious)
        findMenu.addItem("Use Selection for Find", Action(None, 'performFindPanelAction:'), 'cmd+e', tag=const.NSFindPanelActionSetFindString)
        findMenu.addItem("Jump to Selection", Action(None, 'centerSelectionInVisibleArea:'), 'cmd+j')
        spellingMenu = editMenu.addMenu("Spelling")
        spellingMenu.addItem("Spelling...", Action(None, 'showGuessPanel:'), 'cmd+:')
        spellingMenu.addItem("Check Spelling", Action(None, 'checkSpelling:'), 'cmd+;')
        spellingMenu.addItem("Check Spelling as You Type", Action(None, 'toggleContinuousSpellChecking:'))

        windowMenu.addItem("Minimize", Action(None, 'performMinimize:'), 'cmd+m')
        windowMenu.addItem("Zoom", Action(None, 'performZoom:'))
        windowMenu.addSeparator()
        windowMenu.addItem("Bring All to Front", Action(None, 'arrangeInFront:'))

        helpMenu.addItem("%s Help" % appname, Action(NSApp, 'showHelp:'), 'cmd+?')
        
        self.fileMenu = fileMenu
        self.editMenu = editMenu
        self.windowMenu = windowMenu
        self.helpMenu = helpMenu
    



from setuptools import setup

CLASSIFIERS = [
    'Development Status :: 2 - Pre-Alpha',
    'Intended Audience :: Developers',
    'Environment :: Console',
    'License :: OSI Approved :: BSD License',
    'Operating System :: MacOS :: MacOS X',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Objective C',
    'Topic :: Software Development :: Code Generators',
]

LONG_DESC = open('README', 'rt').read() + '\n\n' + open('CHANGES', 'rt').read()

setup(
    name='xibless',
    version='0.1.0',
    author='Virgil Dupras',
    author_email='hsoft@hardcoded.net',
    packages=['xibless'],
    scripts=[],
    install_requires=[],
    url='http://hg.hardcoded.net/xibless/',
    license='BSD',
    description="Generate Objective-C code that builds Cocoa UIs. Replaces XCode's XIBs",
    long_description=LONG_DESC,
    classifiers=CLASSIFIERS,
    entry_points = {
        'console_scripts': [
            'xibless = xibless:main',
        ],
    },
)


from .base import GeneratedItem

class FontFamily(object):
    System = object()
    Label = object()
    Menu = object()
    Menubar = object()
    Message = object()
    Palette = object()
    Titlebar = object()
    Tooltips = object()
    
class FontSize(object):
    System = object()
    SmallSystem = object()
    Label = object()
    RegularControl = object()
    SmallControl = object()
    MiniControl = object()
    
    MethodConsts = {System, SmallSystem, Label}
    ControlConsts = {RegularControl, SmallControl, MiniControl}

class FontTrait(object):
    Bold = object()
    Italic = object()
    # Strikethrough = object() # not supported yet

FAMILY2METHOD = {
    FontFamily.System: 'systemFontOfSize',
    FontFamily.Label: 'labelFontOfSize',
    FontFamily.Menu: 'menuFontOfSize',
    FontFamily.Menubar: 'menuBarFontOfSize',
    FontFamily.Message: 'messageFontOfSize',
    FontFamily.Palette: 'paletteFontOfSize',
    FontFamily.Titlebar: 'titleBarFontOfSize',
    FontFamily.Tooltips: 'toolTipsFontOfSize',
}

SIZE2METHOD = {
    FontSize.System: 'systemFontSize',
    FontSize.SmallSystem: 'smallSystemFontSize',
    FontSize.Label: 'labelFontSize',
}

SIZE2CONTROLCONST = {
    FontSize.RegularControl: 'NSRegularControlSize',
    FontSize.SmallControl: 'NSSmallControlSize',
    FontSize.MiniControl: 'NSMiniControlSize',
}

TRAIT2CONST = {
    FontTrait.Bold: 'NSBoldFontMask',
    FontTrait.Italic: 'NSItalicFontMask',
}

class Font(GeneratedItem):
    OBJC_CLASS = 'NSFont'
    
    def __init__(self, family, size, traits=None):
        # family and size can be a "real" value (str and float) or one of the constants.
        GeneratedItem.__init__(self)
        self.family = family
        self.size = size
        if traits:
            self.traits = set(traits)
        else:
            self.traits = set()
    
    def generateInit(self):
        # We use code blocks to avoid tmp variable name clashes.
        tmpl = GeneratedItem.generateInit(self)
        tmpl.allocinit = """NSFont *$varname$;
            {
                CGFloat _fontSize = $sizeinit$;
                $varname$ = [NSFont $fontinit$];
            }
        """
        if self.family in FAMILY2METHOD:
            tmpl.fontinit = "%s:_fontSize" % FAMILY2METHOD[self.family]
        else:
            tmpl.fontinit = "fontWithName:%s size:_fontSize" % self.family
        if self.size in SIZE2METHOD:
            tmpl.sizeinit = "[NSFont %s]" % SIZE2METHOD[self.size]
        elif self.size in SIZE2CONTROLCONST:
            tmpl.sizeinit = "[NSFont systemFontSizeForControlSize:%s]" % SIZE2CONTROLCONST[self.size]
        else:
            tmpl.sizeinit = str(self.size)
        traitsetup = ""
        for trait in self.traits:
            traitsetup += "[[NSFontManager sharedFontManager] convertFont:%s toHaveTrait:%s];\n" % (self.varname, TRAIT2CONST[trait])
        tmpl.setup = traitsetup
        return tmpl
    



import os.path

from .base import CodeTemplate, KeyValueId, Action, GeneratedItem, owner, NSApp, const
from .view import Pack
from .menu import Menu, MainMenu
from .window import Window
from .button import Button
from .label import Label
from .textfield import TextField
from .font import Font, FontFamily, FontSize, FontTrait

try:
    execfile
except NameError:
    # We're in Python 3
    def execfile(file, globals=globals(), locals=locals()):
        with open(file, "r") as fh:
            exec(fh.read()+"\n", globals, locals)

UNIT_TMPL = """
#import <Cocoa/Cocoa.h>

$classname$* create$name$($ownerclass$ *owner)
{
$contents$
return result;
}
"""

def generate(module_path, dest):
    module_globals = {
        'owner': owner,
        'NSApp': NSApp,
        'const': const,
        'Menu': Menu,
        'MainMenu': MainMenu,
        'Action': Action,
        'Window': Window,
        'Button': Button,
        'Label': Label,
        'TextField': TextField,
        'Font': Font,
        'FontFamily': FontFamily,
        'FontSize': FontSize,
        'FontTrait': FontTrait,
        'Pack': Pack,
    }
    module_locals = {}
    execfile(module_path, module_globals, module_locals)
    assert 'result' in module_locals
    tmpl = CodeTemplate(UNIT_TMPL)
    tmpl.name = os.path.splitext(os.path.basename(dest))[0]
    toGenerate = []
    for key, value in module_locals.items():
        if not isinstance(value, GeneratedItem):
            continue
        value.varname = key
        toGenerate.append(value)
    toGenerate.sort(key=lambda x: x.creationOrder)
    codePieces = []
    for item in toGenerate:
        if item.generated:
            continue
        codePieces.append(item.generate())
    result = module_locals['result']
    tmpl.ownerclass = module_locals.get('ownerclass', 'id')
    tmpl.classname = result.OBJC_CLASS
    tmpl.contents = '\n'.join(codePieces)
    fp = open(dest, 'wt')
    fp.write(tmpl.render())
    fp.close()
